/*
 * coor2.h
 * begin: 02.09.2005
 *
 * copyright (C) 2005 by Jan Ehrenberger < lankvil@seznam.cz >
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

/*
 * C00rs
 * prepracovane coors. matematicky spravne vektory a souradnice
 */

#ifndef C00R_COOR2_H
#define C00R_COOR2_H

namespace c00r
{


	/** coor2
	 * souradnice pro 2D
	 */
	template <class G>
	class coor2 {
	public:
		G x, y;

		/* constructors: */
		coor2 (const G &a, const G &b)
			:x(a), y(b) 
		{ }
		coor2 (const coor2<G> &a)
			:x(a.x), y(a.y)
		{ }
		coor2 ()
			:x(0), y(0)
		{ }
		
		operator bool () const
		{ return x | y; }
		coor2 operator- ()
		{ return coor2<G>(-x, -y); }
	};
	/* porovnavani souradnic */
	template <class G>
	bool operator == (const coor2<G> &a, const coor2<G> &b)
	{ return (a.x == b.x) && (a.y == b.y); }

	template <class G>
	bool operator != (const coor2<G> &a, const coor2<G> &b)
	{ return (a.x != b.x) || (a.y != b.y); }
	
	/* scitani a odcitani -- tady na vektory kaslu, stejne je to jenom pro primitivni 2d veci */
	template <class G>
	coor2<G> operator + (const coor2<G> &a, const coor2<G> &b)
	{ return coor2<G>(a.x + b.x, a.y + b.y); }
	
	template <class G>
	coor2<G> operator - (const coor2<G> &a, const coor2<G> &b)
	{ return coor2<G>(a.x - b.x, a.y - b.y); }

} // namespace c00r

/** makro pro fce typu move(x, y)
 * - jeho pouzitim misto dvou parametru piseme pouze DOUBLEPRM(souradnice)
 */
#define DOUBLEPRM(a)    (a).x, (a).y


#endif // C00R_COOR2_H

