/*
 * coor3.h
 * begin: 01.09.2005
 *
 * copyright (C) 2005 by Jan Ehrenberger < lankvil@seznam.cz >
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

/*
 * C00rs
 * prepracovane coors. matematicky spravne vektory a souradnice
 */

#ifndef C00R_COOR3_H
#define C00R_COOR3_H

#include "vector3.h"

namespace c00r
{


	/** coor3
	 * souradnice pro 3D
	 */
	template <class G>
	class coor3 {
	public:
		G x, y, z;

		/* constructors: */
		coor3 (const G &a, const G &b, const G &c)
			:x(a), y(b), z(c) 
		{ }
		coor3 (const coor3<G> &a)
			:x(a.x), y(a.y), z(a.z)
		{ }
		coor3 ()
			:x(0), y(0), z(0)
		{ }
		
		operator bool () const
		{ return x | y | z; }
		coor3 operator- ()
		{ return coor3<G>(-x, -y, -z); }
	};
	/* porovnavani souradnic */
	template <class G>
	bool operator == (const coor3<G> &a, const coor3<G> &b)
	{ return (a.x == b.x) && (a.y == b.y) && (a.z == a.z); }

	template <class G>
	bool operator != (const coor3<G> &a, const coor3<G> &b)
	{ return (a.x != b.x) || (a.y != b.y) || (a.z != a.z); }
	
	/* soucet souradnice a vektoru -> souradnice */
	template <class G>
	coor3<G> operator + (const coor3<G> &a, const vector3<G> &b)
	{ return coor3<G>(a.x+b.x, a.y+b.y, a.z+b.z); }
	/* rozdil souradnice a vektoru -> souradnice */
	template <class G>
	coor3<G> operator - (const coor3<G> &a, const vector3<G> &b)
	{ return coor3<G>(a.x-b.x, a.y-b.y, a.z-b.z); }


} // namespace c00r

/** makro pro fce typu move(x, y, z)
 * - jeho pouzitim misto dvou parametru piseme pouze TRIPLEPRM(souradnice)
 */
#define TRIPLEPRM(a)    (a).x, (a).y, (a).z


#endif // C00R_COOR3_H

