/*
 * image.cpp
 * begin: 7.6.2006
 *
 * copyright (C) 2006 by Jan Ehrenberger < lankvil@seznam.cz >
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

#include "image.h"

extern "C" {
#include <jpeglib.h>
#include <tiffio.h>
}


/**** JpegImage ****/

/** exception EOpenFailed **/
char *JpegImage::EOpenFailed::_comment = "JpegImage::EOpenFailed: Failed to open file.";

JpegImage::JpegImage (const char *filename)
{
	FILE *file = fopen(filename, "rb");
	if (!file) throw EOpenFailed();

	jpeg_decompress_struct cinfo;
	jpeg_error_mgr err;
	cinfo.err = jpeg_std_error(&err);
	jpeg_create_decompress(&cinfo);
	jpeg_stdio_src(&cinfo, file);
	jpeg_read_header(&cinfo, true);
	jpeg_start_decompress(&cinfo);

	m_width = cinfo.output_width;
	m_height = cinfo.output_height;
	m_pointSize = cinfo.output_components;
	const unsigned row_size = m_width * m_pointSize;
	p_buffer = (GLubyte *)new JSAMPLE[m_height * row_size]; // alokace bufferu
	JSAMPLE *row = (JSAMPLE *)p_buffer;
	while (cinfo.output_scanline < cinfo.output_height) {
		jpeg_read_scanlines(&cinfo, &row, 1);
		row += row_size;
	}
	m_format = GL_RGB;
	switch (cinfo.output_components) {
		case 1: m_format = GL_LUMINANCE; break;
		case 2: m_format = GL_LUMINANCE_ALPHA; break;
		case 3: m_format = GL_RGB; break;
		case 4: m_format = GL_RGBA; break;
	}

}

JpegImage::~JpegImage ()
{
	delete[] p_buffer;
}


/**** TiffImage ****/

/** exception EOpenFailed **/
char *TiffImage::EOpenFailed::_comment = "TiffImage::EOpenFailed: Failed to open file.";

TiffImage::TiffImage (const char *filename)
{
	TIFF *tiff = TIFFOpen(filename, "r");
	if (!tiff) throw EOpenFailed();

	TIFFGetField(tiff, TIFFTAG_IMAGEWIDTH, &m_width);
	TIFFGetField(tiff, TIFFTAG_IMAGELENGTH, &m_height);
	/* predpokladejme ze textura je RGBA */
	m_format = GL_RGBA;
	m_pointSize = 4;
	
	p_buffer = new GLubyte[m_width * m_height * m_pointSize];

	if (!TIFFReadRGBAImage(tiff, m_width, m_height, (uint32 *)p_buffer, 0)) throw h4e3r::EException();

	TIFFClose(tiff);
}

TiffImage::~TiffImage ()
{
	delete[] p_buffer;
}

