/*
 * bus.cpp
 * begin: 3.11.2005
 *
 * copyright (C) 2005 by Jan Ehrenberger < lankvil@seznam.cz >
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

#include "land.h"
#include "texturemanager.h"

#include <iostream>
using namespace std;


//// Land Field ////

void LandField::draw(const unsigned int &a, const unsigned int &b, const unsigned int &w, const unsigned int &h) {
	glBegin(GL_QUADS);
	// 0,0
	glTexCoord2f((float)(a  ) / w, (float)(b) / h);
	glVertex3f(TRIPLEPRM(points[0]));
	glNormal3f(TRIPLEPRM(normals[0])); 
	// 1,0
	glTexCoord2f((float)(a  ) / w, (float)(b+1) / h);
	glVertex3f(TRIPLEPRM(points[1]));
	glNormal3f(TRIPLEPRM(normals[1])); 
	// 1,1
	glTexCoord2f((float)(a+1) / w, (float)(b+1) / h);
	glVertex3f(TRIPLEPRM(points[2]));
	glNormal3f(TRIPLEPRM(normals[2])); 
	// 0,1
	glTexCoord2f((float)(a+1) / w, (float)(b) / h);
	glVertex3f(TRIPLEPRM(points[3]));
	glNormal3f(TRIPLEPRM(normals[3])); 
	glEnd();
	
	/*
	// Zobrazeni normal
	glBegin(GL_LINES);
	glColor3f(1,0,0);
	glVertex3f(TRIPLEPRM(points[0]));
	glVertex3f(TRIPLEPRM(points[0] + normals[0]));
	glVertex3f(TRIPLEPRM(points[1]));
	glVertex3f(TRIPLEPRM(points[1] + normals[1]));
	glVertex3f(TRIPLEPRM(points[2]));
	glVertex3f(TRIPLEPRM(points[2] + normals[2]));
	glVertex3f(TRIPLEPRM(points[3]));
	glVertex3f(TRIPLEPRM(points[3] + normals[3]));
	glEnd();
	glColor3f(0,1,0);
	*/
}

void LandField::computeMainNormal() {
	vector3f a = vector3f(points[0], points[1]);
	vector3f b = vector3f(points[0], points[3]);
	normal = x(a, b);
	normal.normalize();
}


//// computeNormals
// Poradi okolnich ctvercu:
// +---+---+---+
// | 0 | 1 | 2 |
// +---+---+---+
// | 7 |   | 3 |
// +---+---+---+
// | 6 | 5 | 4 |
// +---+---+---+
//
void LandField::computeNormals(
	const LandField &f0,
	const LandField &f1,
	const LandField &f2,
	const LandField &f3,
	const LandField &f4,
	const LandField &f5,
	const LandField &f6,
	const LandField &f7
) {
	normals[0] = normal + f7.normal + f0.normal + f1.normal; normals[0].normalize();
	normals[1] = normal + f1.normal + f2.normal + f3.normal; normals[1].normalize();
	normals[2] = normal + f3.normal + f4.normal + f5.normal; normals[2].normalize();
	normals[3] = normal + f5.normal + f6.normal + f7.normal; normals[3].normalize();
}


//// OLand ////

/** createMap
 * vytvori mapu zeme, predpocita obdelniky, atd...
 */
void OLand::createMap() {
	cout << "OLand::createMap: vytvarim mapu... ";
	p_map = new LandField[(m_width - 1) * (m_height - 1)]; // alokace mapy
	LandField f;
	
	// vytvoreni policek mapy
	for(int a = 0; a < m_width - 1; a++) {
		for(int b = 0; b < m_height - 1; b++) {
			f.points[0] = coor3f(a, (float)p_image->getPoint(a, b) * m_altitude_factor, b);
			f.points[1] = coor3f(a, (float)p_image->getPoint(a, b + 1) * m_altitude_factor, b + 1);
			f.points[2] = coor3f(a + 1, (float)p_image->getPoint(a + 1, b + 1) * m_altitude_factor, b + 1);
			f.points[3] = coor3f(a + 1, (float)p_image->getPoint(a + 1, b) * m_altitude_factor, b);
			f.computeMainNormal();

/*				// 1. trojuhelnik
			f.triangles[0].points[0].x = a * m_field_size;
			f.triangles[0].points[0].z = b * m_field_size;
			f.triangles[0].points[0].y = p_image->getPoint(a, b) * m_altitude_factor;
			f.triangles[0].points[1].x = (a + 1) * m_field_size;
			f.triangles[0].points[1].z = b * m_field_size;
			f.triangles[0].points[1].y = p_image->getPoint(a + 1, b) * m_altitude_factor;
			f.triangles[0].points[2].x = a * m_field_size;
			f.triangles[0].points[2].z = (b + 1) * m_field_size;
			f.triangles[0].points[2].y = p_image->getPoint(a, b + 1) * m_altitude_factor;
			f.triangles[0].computeNormal();
			// 2. trojuhelnik
			f.triangles[1].points[0].x = a * m_field_size;
			f.triangles[1].points[0].z = (b + 1) * m_field_size;
			f.triangles[1].points[0].y = p_image->getPoint(a, b + 1) * m_altitude_factor;
			f.triangles[1].points[1].x = (a + 1) * m_field_size;
			f.triangles[1].points[1].z = b*m_field_size;
			f.triangles[1].points[1].y = p_image->getPoint(a + 1, b) * m_altitude_factor;
			f.triangles[1].points[2].x = (a + 1) * m_field_size;
			f.triangles[1].points[2].z = (b + 1) * m_field_size;
			f.triangles[1].points[2].y = p_image->getPoint(a + 1, b + 1) * m_altitude_factor;
			f.triangles[1].computeNormal();
*/				
			map(a, b) = f;
		}
	}
	computeNormals();
	cout << "mapa hotova." << endl;
}


void OLand::computeNormals()
{
	for(int a = 1; a < m_width - 2; a++) {
		for(int b = 1; b < m_height - 2; b++) {
			// Spocitame normaly vzhledem k okolnim polickum
			map(a, b).computeNormals(
				map(a - 1, b - 1),
				map(a    , b - 1),
				map(a + 1, b - 1),
				map(a + 1, b    ),
				map(a + 1, b + 1),
				map(a    , b + 1),
				map(a - 1, b + 1),
				map(a - 1, b    )
			);
		}
	}
}

/** drawMap
 * Vykresli mapu na scenu
 */
void OLand::drawMap() {

	glBegin(GL_TRIANGLES);
	glColor3f(0.4, 0.8, 0.4);
	for(int a = 1; a < m_width - 2; a++) {
		for(int b = 1; b < m_height - 2; b++) {
			p_map[(a * (m_width - 1)) + b].draw(a, b, m_width, m_height);
		}
	}
	glEnd();
}


OLand::OLand (const char *height_map, const char *texture, unsigned int field_size, float altitude_factor)
: m_field_size(field_size), m_altitude_factor(altitude_factor), m_texture(-1)
{
	p_image = new JpegImage(height_map);
	m_width = p_image->getWidth();
	m_height = p_image->getHeight();

	if(texture)
		m_texture = g_texMan.getFileTex(texture);

	createMap();
	
	// Vytvorime display list
	m_displayList = glGenLists(1);
	glNewList(m_displayList, GL_COMPILE);
	drawMap();
	glEndList();
}

OLand::~OLand () {
	delete p_image;
}

void OLand::draw () {
	if(m_texture >= 0)
		glBindTexture(GL_TEXTURE_2D, m_texture);
	else
		glDisable(GL_TEXTURE_2D);
	
	glCallList(m_displayList);

	if(m_texture < 0)
		glEnable(GL_TEXTURE_2D);

}
