/*
 * matrix.cpp
 * begin: 02.09.2005
 *
 * copyright (C) 2005 by Jan Ehrenberger < lankvil@seznam.cz >
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

#include "matrix.h"
#include <cmath>
#include <iostream>

using namespace std;

Matrix operator*(const Matrix& a, const Matrix& b)
{
  Matrix ret;
  for( int i=0; i<4; i++ )
    for( int k=0; k<4; k++ )
    {
      ret.m_data[4*i+k] = 0;
      for( int j=0; j<4; j++ )
        ret.m_data[4*i+k] += a.m_data[4*i+j] * b.m_data[4*j+k];
    }
  return ret;
}

/** operator negace */
Matrix Matrix::operator-() const
{
  Matrix ret(
    -m_data[0], -m_data[1], -m_data[2], m_data[3],
    -m_data[4], -m_data[5], -m_data[6], m_data[7],
    -m_data[8], -m_data[9], -m_data[10], m_data[11],
    -m_data[12], -m_data[13], -m_data[14], m_data[15]
  );
  return ret;
}

/* translation - posunuti */
Matrix& Matrix::translation(const coor3f& a)
{
  Matrix mat(
    1, 0, 0, 0,
    0, 1, 0, 0,
    0, 0, 1, 0,
    a.x, a.y, a.z, 1);
  return multiply(mat);
}

/* scaling - zmena velikosti */
Matrix& Matrix::scaling(const coor3f& a)
{
  Matrix mat(
    a.x, 0,   0,   0,
    0,   a.y, 0,   0,
    0,   0,   a.z, 0,
    0,   0,   0,   1);
  return multiply(mat);
}

/* rotation */
Matrix& Matrix::rotation(const GLfloat& angle, const coor3f& axis)
{
  const GLfloat cosx = cos(angle*axis.x);
  const GLfloat sinx = sin(angle*axis.x);
  const GLfloat cosy = cos(angle*axis.y);
  const GLfloat siny = sin(angle*axis.y);
  const GLfloat cosz = cos(angle*axis.z);
  const GLfloat sinz = sin(angle*axis.z);

  Matrix mat(cosy*cosz, -sinz, siny, 0,
             sinz, cosx*cosz, -sinx, 0,
             -siny, sinx, cosx*cosy, 0,
              0, 0, 0, 1);
  return multiply(mat);
}

/* rotx - rotace kolem osy x objektu */
Matrix& Matrix::rotx(const GLfloat& angle)
{
  const GLfloat ccos = cos(angle);
  const GLfloat csin = sin(angle);
  Matrix mat(1,    0,     0, 0,
             0, ccos, -csin, 0,
             0, csin,  ccos, 0,
             0,    0,     0, 1);
  return multiply(mat);
}

/* roty - rotace kolem osy y objektu */
Matrix& Matrix::roty(const GLfloat& angle)
{
  const GLfloat ccos = cos(angle);
  const GLfloat csin = sin(angle);
  Matrix mat(ccos, 0, csin, 0,
                0, 1,    0, 0,
            -csin, 0, ccos, 0,
                0, 0,    0, 1);
  return multiply(mat);
}

/* rotz - rotace kolem osy z objektu */
Matrix& Matrix::rotz(const GLfloat& angle)
{
  const GLfloat ccos = cos(angle);
  const GLfloat csin = sin(angle);
  Matrix mat(ccos, -csin, 0, 0,
             csin,  ccos, 0, 0,
                0,     0, 1, 0,
                0,     0, 0, 1);
  return multiply(mat);
}

/* vrati determinant matice */
GLfloat Matrix::determinant() const
{
/*
0  1  2  3
4  5  6  7
8  9  10 11
12 13 14 15
*/
  GLfloat D;
  D = m_data[0]*m_data[5]*m_data[10];
  D += m_data[1]*m_data[6]*m_data[8];
  D += m_data[2]*m_data[4]*m_data[9];
  D -= m_data[2]*m_data[5]*m_data[8];
  D -= m_data[1]*m_data[4]*m_data[10];
  D -= m_data[6]*m_data[9]*m_data[0];
  return D;
}

void Matrix::invert()
{
  for (int i=1; i < 4; i++)
    m_data[i] /= m_data[0]; // normalize row 0
  for (int i=1; i < 4; i++)  {
    for (int j=i; j < 4; j++)  { // do a column of L
      GLfloat sum = 0.0;
      for (int k = 0; k < i; k++)
        sum += m_data[j*4+k] * m_data[k*4+i];
      m_data[j*4+i] -= sum;
  }
  if (i == 4-1) continue;

  for (int j=i+1; j < 4; j++)  {  // do a row of U
     GLfloat sum = 0.0;
     for (int k = 0; k < i; k++)
       sum += m_data[i*4+k]*m_data[k*4+j];
     m_data[i*4+j] = (m_data[i*4+j]-sum) / m_data[i*4+i];
     }
  }

  for ( int i = 0; i < 4; i++ )  // invert L
     for ( int j = i; j < 4; j++ )  {
       GLfloat x = 1.0;
       if ( i != j ) {
         x = 0.0;
         for ( int k = i; k < j; k++ )
           x -= m_data[j*4+k]*m_data[k*4+i];
       }
       m_data[j*4+i] = x / m_data[j*4+j];
     }

  for ( int i = 0; i < 4; i++ )   // invert U
    for ( int j = i; j < 4; j++ )  {
      if ( i == j ) continue;
      GLfloat sum = 0.0;
      for ( int k = i; k < j; k++ )
         sum += m_data[k*4+j]*( (i==k) ? 1.0 : m_data[i*4+k] );
      m_data[i*4+j] = -sum;
    }

  for ( int i = 0; i < 4; i++ )   // final inversion
   for ( int j = 0; j < 4; j++ )  {
     GLfloat sum = 0.0;
     for ( int k = ((i>j)?i:j); k < 4; k++ )
       sum += ((j==k)?1.0:m_data[j*4+k])*m_data[k*4+i];
     m_data[j*4+i] = sum;
   }
}

void Matrix::print() const
{
  cout << "{" << endl;
  cout << "{ " << m_data[0] << "; " << m_data[1] << "; " << m_data[2] << "; " << m_data[3] << " }" << endl;
  cout << "{ " << m_data[4] << "; " << m_data[5] << "; " << m_data[6] << "; " << m_data[7] << " }" << endl;
  cout << "{ " << m_data[8] << "; " << m_data[9] << "; " << m_data[10] << "; " << m_data[11] << " }" << endl;
  cout << "{ " << m_data[12] << "; " << m_data[13] << "; " << m_data[14] << "; " << m_data[15] << " }" << endl;
  cout << "}" << endl;
}

coor3f Matrix::getPos() const {
	return coor3f(m_data[12], m_data[13], m_data[14]);
}
