/*
 * matrix.h
 * begin: 02.09.2005
 *
 * copyright (C) 2005 by Jan Ehrenberger < lankvil@seznam.cz >
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

#ifndef BUSIM_MATRIX_H
#define BUSIM_MATRIX_H

#include "head.h"

class Matrix
{
  GLfloat m_data[16];

  Matrix& multiply(const Matrix& m) // pro nahrazeni pro tridu CameraMatrix
  {
    return *this = m * (*this);
  }

public:

  friend Matrix operator*(const Matrix&, const Matrix&);

  Matrix()
  {
    m_data[0] = 1; m_data[1] = 0; m_data[2] = 0; m_data[3] = 0;
    m_data[4] = 0; m_data[5] = 1; m_data[6] = 0; m_data[7] = 0;
    m_data[8] = 0; m_data[9] = 0; m_data[10] = 1; m_data[11] = 0;
    m_data[12] = 0; m_data[13] = 0; m_data[14] = 0; m_data[15] = 1;
  }
  Matrix(GLfloat m0, GLfloat m1, GLfloat m2, GLfloat m3,
         GLfloat m4, GLfloat m5, GLfloat m6, GLfloat m7,
         GLfloat m8, GLfloat m9, GLfloat m10, GLfloat m11,
         GLfloat m12, GLfloat m13, GLfloat m14, GLfloat m15)
  {
    m_data[0] = m0; m_data[1] = m1; m_data[2] = m2; m_data[3] = m3;
    m_data[4] = m4; m_data[5] = m5; m_data[6] = m6; m_data[7] = m7;
    m_data[8] = m8; m_data[9] = m9; m_data[10] = m10; m_data[11] = m11;
    m_data[12] = m12; m_data[13] = m13; m_data[14] = m14; m_data[15] = m15;
  }

  void apply()
    { glMultMatrixf(m_data); }

  void invert();

  coor3f getPos() const;

  Matrix& translation(const coor3f&);
  Matrix& scaling(const coor3f&);
  Matrix& rotation(const GLfloat& angle, const coor3f& axis);
  Matrix& rotx(const GLfloat& angle);
  Matrix& roty(const GLfloat& angle);
  Matrix& rotz(const GLfloat& angle);
  GLfloat determinant() const;
  void print() const;

  const GLfloat& operator[](const unsigned char& n) const
    { return m_data[n]; }
  GLfloat& operator[](const unsigned char& n)
    { return m_data[n]; }
  Matrix operator-() const;
};

#endif // BUSIM_MATRIX_H

