/*
 * object.h
 * begin: 08.10.2005
 *
 * copyright (C) 2005 by Jan Ehrenberger < lankvil@seznam.cz >
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

#ifndef BUSIM_OBJECT_H
#define BUSIM_OBJECT_H

#include "head.h"
#include "matrix.h"
#include <list>


// OObject
// - uplne nejzakladnejsi objekt
class OObject
{
  static unsigned long int m_objectCounter;

  friend class OMultiple;
  OObject *m_pParent;
  bool m_alive; // urcuje, jestli je objekt zivy, nebo mrtvy. pokud je mrtvy, bude co nejdriv odstranen

public:
//	Matrix m_matrix;

  OObject()
    :m_pParent(NULL), m_alive(true)
  { m_objectCounter++; }
  virtual ~OObject()
  { m_objectCounter--; }

  OObject *getParent() const
  { return m_pParent; }
  
  void die()
  { m_alive = false; }

  virtual void update()
  { }
  virtual void draw() = 0;

  static unsigned long int numberOfObjects()
  { return m_objectCounter; }
};


typedef std::list<OObject *> objectList;


// OMultiple
// - objekt, ktery se sklada z dalsich objektu
class OMultiple
:public OObject
{
  objectList m_objects;

public:
  OMultiple()
    { }
  ~OMultiple();

  void add(OObject *o)
    { o->m_pParent = this; m_objects.push_back(o); }

  void draw();
  void update();
};

// OExternal
// - objekt nahravany ze souboru
class OExternal
:public OObject
{
	objectList m_objects;

	void _draw()
	{ }
public:
	OExternal(const char *);
};


#endif // BUSIM_OBJECT_H

