/*
 * scene.cpp
 * begin: 08.10.2005
 *
 * copyright (C) 2005 by Jan Ehrenberger < lankvil@seznam.cz >
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

#include "scene.h"
#include "text.h"
#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL/SDL.h>
#include <math.h>

Scene g_scene;

extern bool keyb[SDLK_LAST];


Scene::~Scene()
{
	// zniceni fontu
	if (g_font != NULL)
		delete g_font;
	g_font == NULL;
}

void Scene::init()
{
	glShadeModel(GL_SMOOTH);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glShadeModel(GL_SMOOTH);
	// depth buffer
	glEnable(GL_DEPTH_TEST);
	// textury	
	glEnable(GL_TEXTURE_2D);
//	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
//	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
//	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	
	glPointSize(3.0);

	glPolygonMode(GL_FRONT, GL_FILL);
	glPolygonMode(GL_BACK, GL_LINE);

	// svetylka
	float lightAmbient[] = { 0.5f, 0.5f, 0.5f, 0.0f };
	float lightDiffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
	float lightPos[] = { 2.0f, 5.0f, 0.0f, 1.0f };
	glLightfv(GL_LIGHT1, GL_AMBIENT, lightAmbient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, lightDiffuse);
	glLightfv(GL_LIGHT1, GL_POSITION, lightPos);
	glEnable(GL_LIGHT1);
	glEnable(GL_LIGHTING);


	// vytvoreni fontu
	g_font = new BuFont("-adobe-courier-bold-r-*-*-14-*-*-*-*-*-*-2");
}

void Scene::setFog(const Color& col, const GLfloat& density)
{
	glEnable(GL_FOG); // povolit mlhu
	glFogi(GL_FOG_MODE, GL_EXP);
	glFogfv(GL_FOG_COLOR, (GLfloat *)&col);
	glFogf(GL_FOG_DENSITY, density); // hustota mlhy
	glHint(GL_FOG_HINT, GL_DONT_CARE);
	glFogf(GL_FOG_START, 1.0); // pouzito pro linearni funkci, tady zbytecne
	glFogf(GL_FOG_END, 5.0);
	m_bgColor = col; // barva pozadi by mela mit barvu mlhy
}

/* zmeni velikost sceny
 * mela by se volat po zmene velikosti okna
 */
void Scene::resize(const coor2n& size)
{
	glViewport(0, 0, DOUBLEPRM(size));

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, (GLdouble)size.x/(GLdouble)size.y, 0.1f, m_viewDistance);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

/** checkKeyb
 * zkontroluje, jestli nekdo nezmack klavesu, a kdyz jo, tak neco udela
 */
void Scene::checkKeyb()
{
	if (keyb[SDLK_w])
		m_camera.translation(coor3f(0, 0, 1));
	if (keyb[SDLK_s])
		m_camera.translation(coor3f(0, 0, -1));
	if (keyb[SDLK_a])
		m_camera.translation(coor3f(1, 0, 0));
	if (keyb[SDLK_d])
		m_camera.translation(coor3f(-1, 0, 0));
	if (keyb[SDLK_UP])
		m_spectator.translation(coor3f(0, 0, -0.3));
	if (keyb[SDLK_DOWN])
		m_spectator.translation(coor3f(0, 0, 0.3));
	if (keyb[SDLK_LEFT])
		m_spectator.roty(-0.01);
	if (keyb[SDLK_RIGHT])
		m_spectator.roty(0.01);
	
	if (keyb[SDLK_p] && keyb[SDLK_LALT]) {
		/* zmena vykreslovani (plny povrch / cary) */
		if (m_polygonMode == GL_FILL) m_polygonMode = GL_LINE;
		else m_polygonMode = GL_FILL;
		glPolygonMode(GL_FRONT, m_polygonMode);
	}

}

/** mouseMove
 * zareaguje na udalost pohybu mysi
 */
void Scene::mouseMove(const coor2n& change)
{
	m_spectator.rotx(-change.y / 1000.0);
	m_spectator.roty(-change.x / 1000.0);
}

/** update scenu */
void Scene::update()
{
	checkKeyb();
	OMultiple::update();
}

/* vypocet FPS - volat pokazde pri renderovani */
void Scene::calculateFPS()
{
	unsigned int now = SDL_GetTicks();
	m_fps = 1000.0 / (now - m_lastRender);
	m_lastRender = now;
}

void DrawGrid()
{
	glColor4f(0.0f, 1.0f, 0.0f, 0.5f);

	// Draw a 1x1 grid along the X and Z axis'
	for(double Aa = -200; Aa <= 200; Aa+=2)
	{
		glBegin(GL_LINES);
		// Do the horizontal lines (along the X)
		glVertex3f(-200, 0, Aa);
		glVertex3f(200, 0, Aa);

		// Do the vertical lines (along the Z)
		glVertex3f(Aa, 0, -200);
		glVertex3f(Aa, 0, 200);
		glEnd();
	}
}

void Scene::render()
{
	coor3f pos = m_spectator.getPos();

	calculateFPS();
	glClearColor(m_bgColor.r, m_bgColor.g, m_bgColor.b, m_bgColor.a);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);

	
	m_camera.apply();
	
	Matrix m = m_spectator;
	m.invert();
	m.apply();


//	glDisable(GL_TEXTURE_2D);
//	DrawGrid();
	glEnable(GL_TEXTURE_2D);
	OMultiple::draw();

//	glTranslatef(TRIPLEPRM(p_vehicle->m_pos));
	glLoadIdentity(); // musi byt zde aby se pak text vykresloval pres vsechno ostatni

	g_font->buf_printf(0, "Hello World!!!");
	g_font->buf_printf(1, "scene FPS: %d", (int)m_fps);
	g_font->buf_printf(2, "pos x=%4.2f y=%4.2f z=%4.2f", TRIPLEPRM(pos));
	glTranslatef(-0.1, 0.35, 0);
	g_font->begin();
	g_font->print_buf();
	g_font->end();
}
