/*
 * sdlstuff.cpp
 * begin: 02.09.2005
 *
 * copyright (C) 2005 by Jan Ehrenberger < lankvil@seznam.cz >
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

#include "sdlstuff.h"
#include "scene.h"
#include <iostream>

using namespace std;


coor2n g_screenSize(800, 600);
const int g_screenDepth = 16;
bool keyb[SDLK_LAST];


SDLStuff::SDLStuff()
{
	if (SDL_Init(SDL_INIT_VIDEO) < 0)
	{
		std::cerr << "failed initializing SDL Video: " << SDL_GetError() << std::endl;
		exit(13);
	}
}

SDLStuff::~SDLStuff()
{
	SDL_Quit(); // shut down all the SDL stuff
}

void SDLStuff::setupPixelFormat()
{
	const SDL_VideoInfo* VideoInfo = SDL_GetVideoInfo(); /* query SDL for information about our video hardware */
	if (VideoInfo == NULL) // if this query fails
	{
		std::cerr << "Failed getting Video Info : " << SDL_GetError() << std::endl; // report error
		exit(13);
	}

	m_videoFlags = SDL_OPENGL | SDL_HWPALETTE | SDL_RESIZABLE;
	if(VideoInfo->hw_available) /* is it a hardware surface */
	m_videoFlags |= SDL_HWSURFACE;
	else
	m_videoFlags |= SDL_SWSURFACE;
	if(VideoInfo->blit_hw)
	m_videoFlags |= SDL_HWACCEL;

	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, g_screenDepth);
	SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 0);
	SDL_GL_SetAttribute( SDL_GL_ACCUM_RED_SIZE, 0);
	SDL_GL_SetAttribute( SDL_GL_ACCUM_GREEN_SIZE, 0);
	SDL_GL_SetAttribute( SDL_GL_ACCUM_BLUE_SIZE, 0);
	SDL_GL_SetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, 0);
}

/* pohyb pomoci mysi */
void SDLStuff::mouseStuff()
{
	coor2n pos;

	Uint8 buttons = SDL_GetMouseState(&pos.x, &pos.y);
	if ((SDL_BUTTON(buttons) != SDL_BUTTON_LEFT)) return;
	if (m_lastMouse == pos) return;

	g_scene.mouseMove(m_lastMouse - pos);
	SDL_WarpMouse(DOUBLEPRM(m_lastMouse));
}

void SDLStuff::wndInit(const char* title)
{
	setupPixelFormat();
	m_pSurface = SDL_SetVideoMode(g_screenSize.x, g_screenSize.y, g_screenDepth, m_videoFlags);
	if (m_pSurface == NULL)
	{
		std::cerr << "Failed to Create Window : " << SDL_GetError() << std::endl;
		exit(13);
	}

	SDL_WM_SetCaption(title, title); /* set the window caption and icon caption */
}

/* main loop */
void SDLStuff::loop()
{
	SDL_Event event;
	bool done = false;

	while (!done) // main loop looping around for ever
	{
		while (SDL_PollEvent(&event))
		{
			switch (event.type)
			{
			case SDL_QUIT:
				done = true;
				break;

			case SDL_KEYDOWN:
				switch (event.key.keysym.sym)
				{
				case SDLK_ESCAPE:
					done = true;
					break;
				default:
					keyb[event.key.keysym.sym] = true;
				}
				break;

			case SDL_KEYUP:
				keyb[event.key.keysym.sym] = false;
				break;

			case SDL_MOUSEBUTTONDOWN:
				SDL_GetMouseState(&m_lastMouse.x, &m_lastMouse.y);
				break;

			case SDL_VIDEORESIZE :
				m_pSurface = SDL_SetVideoMode(event.resize.w, event.resize.h,
				g_screenDepth, m_videoFlags);
				if (m_pSurface == NULL)
				{
					std::cerr << "failed resizing SDL window: " << SDL_GetError() << std::endl;
					exit(13);
				}
				g_screenSize.x = event.resize.w;
				g_screenSize.y = event.resize.h;
				g_scene.resize(g_screenSize);
				break;
			} // switch
		} // while (SDL_ ...

		unsigned int begin = SDL_GetTicks();

		mouseStuff();
		g_scene.update();
//		cout << "update: " << SDL_GetTicks() - begin << "\t\t";

		unsigned int render_begin = SDL_GetTicks();
		g_scene.render();
//		cout << "render: " << SDL_GetTicks() - render_begin << endl;

//		cout << "FPS: " << g_scene.getFPS() << endl;

		unsigned int diff = SDL_GetTicks()-begin;
		//    cout << "diff: " << diff << endl;
//		if (diff < 20) {
//			cout << "delay: " << 20-diff << endl;
//			SDL_Delay(20-diff);
//		}

		SDL_GL_SwapBuffers();
	} // while( !done )
}
