/*
 * texturemanager.cpp
 * begin: 11.10.2005
 *
 * copyright (C) 2005 by Jan Ehrenberger < lankvil@seznam.cz >
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

#include "texturemanager.h"
#include "image.h"

TextureManager g_texMan;

TextureManager::TextureManager()
{ }
TextureManager::~TextureManager()
{ }

/*** exception OpenFailed ***/
char *TextureManager::EOpenFailed::_comment = "TextureManager failed to open file";

/*** exception UnknownFormat ***/
char *TextureManager::EUnknownFormat::_comment = "TextureManager doesn't know format of some file";


#ifdef SIMPLE3D_WINDOWS /*** code for windows ***/

/** getFileTex
 * naloaduje texturu a vrati jeji cislo. pokud byla textura naloadovana uz nekdy driv,
 * pak jenom vyhleda jeji cislo v seznamu
 */
GLuint TextureManager::getFileTex(const char *n, format t)
throw (h4e3r::EException, EOpenFailed, EUnknownFormat)
{
	/* zatim nic nedela */
	return 0;
}

#else /*** code for linux ***/

extern "C" {
  #include <stdio.h>
  #include <tiffio.h>
  #include <jpeglib.h>
  #include <string.h>
}

/** getFileTex
 * naloaduje texturu a vrati jeji cislo. pokud byla textura naloadovana uz nekdy driv,
 * pak jenom vyhleda jeji cislo v seznamu
 */
GLuint TextureManager::getFileTex(const char *n, format t)
throw (h4e3r::EException, EOpenFailed, EUnknownFormat)
{
	map_t::iterator iA;
	if ((iA = m_map.find(n)) != m_map.end()) return iA->second; /* pokud textura uz existuje ... */

	Image *i; // nacteny obrazek
	GLuint texID; // identifikator textury

	switch (t) {
		case tiff: i = new TiffImage(n); break;
		case jpeg: i = new JpegImage(n); break;
		default: throw EUnknownFormat();
	}

	glGenTextures(1, &texID);
	glBindTexture(GL_TEXTURE_2D, texID);
	glTexImage2D(GL_TEXTURE_2D, 0, i->getPointSize(), i->getWidth(), i->getHeight(), 0, i->getFormat(), GL_UNSIGNED_BYTE, (GLvoid *)i->p_buffer);
	delete i;

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT); // opakovani textury
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); // volba filtru
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	m_map[n] = texID;
	return texID;
}

#endif // SIMPLE3D_WINDOWS

/** getFileTex
 * typ textury urcuje podle pripony
 */
GLuint TextureManager::getFileTex(const char *n)
throw (h4e3r::EException, EOpenFailed, EUnknownFormat)
{
  const char *p = strrchr(n, '.'); // hleda posledni tecku, za niz bude s nejvetsi pravdepodobnosti pripona
  if (!p) throw EUnknownFormat();
  format f;
  if (strcmp(p, ".tiff") == 0) f = tiff;
  else if (strcmp(p, ".jpg") == 0 || strcmp(p, ".jpeg") == 0) f = jpeg;
  else throw EUnknownFormat();
  return getFileTex(n, f);
}
